function [rlt_y, rlt_h, rlt_lambda, iter_times] = ...
    solveMVC_SL(eps_stop, Q_info, gamma_reg, b_bal)

%% Initialization
n = size(Q_info,1);
I = eye(n,n);

[V D] = eigs(Q_info,2,'sa');
[ds,di] = sort(diag(D));
v2 = V(:,di(2));
h0 = sign(v2-mean(v2))/sqrt(n);
lam0 = 0;

%% Iteration
iter_times = 0;

h_pre = Inf;
lam_pre = Inf;
h_cur = h0; 
lam_cur = lam0;

cvx_quiet(true)
while norm(h_cur-h_pre,2)+(lam_cur-lam_pre)^2 > eps_stop
    h_pre = h_cur;
    lam_pre = lam_cur;
    % disp(['Iteration: ',int2str(iter_times),'...']);
    
    if b_bal>=0&&b_bal<=sqrt(n)
        cvx_begin
            variable p(n);
            dual variable lam;
            minimize( p'*(gamma_reg*Q_info-lam_pre*I)*p + 2*(gamma_reg*Q_info*h_pre-sign(h_pre))'*p )
            subject to
                2*h_pre'*p + h_pre'*h_pre == 1 : lam;
                sum(p)+sum(h_pre) <= b_bal;
                sum(p)+sum(h_pre) >= -b_bal;
        cvx_end
    
        if (sum(isnan(p))>0)
            disp('Warning: the class balance parameter is too small!');
            b_bal = b_bal*2;
        else
            h_cur = h_pre+p;
            lam_cur = lam;
            iter_times = iter_times+1;
        end
    else
        cvx_begin
            variable p(n);
            dual variable lam;
            minimize( p'*(gamma_reg*Q_info-lam_pre*I)*p + 2*(gamma_reg*Q_info*h_pre-sign(h_pre))'*p )
            subject to
                2*h_pre'*p + h_pre'*h_pre == 1 : lam;
        cvx_end
        h_cur = h_pre+p;
        lam_cur = lam;
        iter_times = iter_times+1;
    end
end
cvx_quiet(false)

%% Finalization
rlt_y = sign(h_cur);
rlt_h = h_cur;
rlt_lambda = lam_cur;
